import es
import time
import path

# Addon info
info			= es.AddonInfo()
info.name		= "Command Logger"
info.version	= "1.1"
info.url		= "http://addons.eventscripts.com/addons/view/cmd_logger"
info.author		= "Ojii"
info.basename	= "cmd_logger"

# Paths
serverdir = path.path(str(es.ServerVar('eventscripts_gamedir')))
logdir = serverdir.joinpath('logs')
if not logdir.isdir():
    logdir.mkdir()
mylogdir = logdir.joinpath('cmdlogs')
if not mylogdir.isdir():
    mylogdir.mkdir()
cfgdir = serverdir.joinpath('cfg')

# Utils

def truncate(string, maxlength):
    if len(string) <= maxlength:
        return string
    return string[:maxlength-3] + '...'
    
def lines_comment_strip(buf):
    """
    Strip comments from a buffer.
    """
    if isinstance(buf, basestring):
        return map(lambda x: x.split('//')[0].strip(), buf.split('\n'))
    return buf
    
def get_cfg(filename):
    thefile = cfgdir.joinpath(filename)
    thefile.touch()
    lines = filter(bool, map(lines_comment_strip, thefile.lines(retain=False)))
    return lines

# Logger

class Logger(object):
    timeformat = '%Y-%m-%d %H:%M:%S'
    dateformat = '%Y-%m-%d'
    logformat = '%(time)s | %(command)-35s | %(steamid)s\n'
    filename = '%(date)s.txt'
    def __init__(self, client_command_list, say_command_list, any_mode=False):
        es.addons.registerSayFilter(self.say_filter)
        es.addons.registerClientCommandFilter(self.client_filter)
        self.client_command_list = client_command_list
        self.say_command_list = say_command_list
        self.any_mode = any_mode
        self.file = None
        self.open()
        
    def open(self):
        self.close()
        filename = self.filename % {'date': time.strftime(self.dateformat)}
        myfile = mylogdir.joinpath(filename)
        if myfile.isfile():
            self.file = myfile.open('a')
        else:
            self.file = myfile.open('w')
            self.write_headers()
            
    def write_headers(self):
        self._write('Say- & Clientcommand Logs from %s\n' % time.strftime(self.dateformat))
        self._write('\n')
        self._write('Timestamp           | Command                             | SteamID\n')
        self._write('--------------------|-------------------------------------|---------------------\n')
        
    def say_filter(self, userid, message, teamonly):
        text = message.strip('"')
        args = text.split(' ')
        if args[0] in self.say_command_list or self.any_mode:
            self.log(userid, args)
        return (userid, text, teamonly)
    
    def client_filter(self, userid, args):
        if args[0] in self.client_command_list or self.any_mode:
            self.log(userid, args)
        return True
            
    def log(userid, args):
        steamid = es.getplayersteamid(userid)
        command = ' '.join(args)
        timestamp = time.strftime(self.timeformat)
        data = {
            'time': timestamp,
            'command': truncate(command, 35),
            'steamid': steamid,
        }
        self._write(self.logformat % data)
        
    def _write(self, data):
        if self.file:
            self.file.write(data)
            
    def close(self):
        if self.file:
            self.file.close()
        self.file = None
        
    def __del__(self):
        es.addons.unregisterSayFilter(self.say_filter)
        es.addons.unregisterClientCommandFilter(self.client_filter)
        self.close()
logger = Logger(get_cfg('cmd_logger_saycommands.txt'), get_cfg('cmd_logger_clientcommands.txt'))

def es_map_start(ev):
    logger.close()
    logger.open()
    
def unload():
    del logger
